@@ -8,7 +8,7 @@ from .grid import *
 
 
 class InstructionType(Enum):
-    '''Represents the various types of SpaceChem non-arrow instruction.'''
+    """Represents the various types of SpaceChem non-arrow instruction."""
     START = 'S'
     INPUT = 'i'
     OUTPUT = 'o'
@@ -36,10 +36,10 @@ class InstructionType(Enum):
 class Instruction(namedtuple('Instruction', ('type', 'direction', 'target_idx'),
                              # Default direction and target_idx if unneeded
                              defaults=(None, None))):
-    '''Represents a non-arrow SpaceChem instruction and any associated properties.
+    """Represents a non-arrow SpaceChem instruction and any associated properties.
     Direction is used by rotates, flip-flops and sensor cmds. Target index represents either the zone idx for an input
     or output instruction, or an element's atomic # in the case of a sensor cmd.
-    '''
+    """
     __slots__ = ()
 
     STR_MAP = {InstructionType.START: 'S',
@@ -142,9 +142,9 @@ class Waldo:
         self.is_rotating = False  # Used to distinguish the first vs second cycle on a rotate command
 
     def __hash__(self):
-        '''A waldo's run state is uniquely identified by its position, direction,
+        """A waldo's run state is uniquely identified by its position, direction,
         whether it's holding a molecule, whether it's rotating, and its flip-flop states.
-        '''
+        """
         return hash((self.position, self.direction, self.molecule is None, self.is_rotating,
                      tuple(self.flipflop_states.values())))
 
@@ -159,11 +159,12 @@ class Waldo:
                        and not (isinstance(instr, Instruction) and instr.type == InstructionType.START)))
 
     def cur_cmd(self):
-        '''Return a waldo's current 'command', i.e. non-arrow instruction, or None.'''
+        """Return a waldo's current 'command', i.e. non-arrow instruction, or None."""
         return None if self.position not in self.instr_map else self.instr_map[self.position][1]
 
     def export_str(self):
-        '''Represent this waldo's instructions in solution export string format.'''
+        """Represent this waldo's instructions in solution export string format."""
+        start_line = None
         lines = []
         for posn, (arrow, instr) in self.instr_map.items():
             if arrow is not None:
@@ -172,9 +173,13 @@ class Waldo:
                 lines.append(f"MEMBER:'instr-arrow',{dirn_degrees},0,{waldo_int},{posn.col},{posn.row},0,0")
 
             if instr is not None:
-                lines.append(instr.export_str(waldo_idx=self.idx, posn=posn))
+                if instr.type == InstructionType.START:
+                    start_line = instr.export_str(waldo_idx=self.idx, posn=posn)
+                else:
+                    lines.append(instr.export_str(waldo_idx=self.idx, posn=posn))
 
-        return '\n'.join(lines)
+        # Put the start instr line at the front and sort the remaining lines
+        return start_line + '\n' + '\n'.join(sorted(lines))
 
     def trace_path(self, num_cols=10, num_rows=8):
         """Return a dict of position: directions representing paths this waldo visually traces.
@@ -183,7 +188,7 @@ class Waldo:
         def is_valid_posn(posn):
             return 0 <= posn.col < num_cols and 0 <= posn.row < num_rows
 
-        branching_instr_types = set((InstructionType.SENSE, InstructionType.FLIP_FLOP))
+        branching_instr_types = {InstructionType.SENSE, InstructionType.FLIP_FLOP}
 
         # Override the start direction with any arrow since unlike other directional commands it can't branch
         start_posn, start_dirn = next((posn, arrow_dirn if arrow_dirn is not None else cmd.direction)
